import _extends from 'babel-runtime/helpers/extends';
import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import uuid from 'uuid/v1';
import { debounce } from 'lodash';
import PubSub from '../PubSub';
import FieldsModel from './FieldsModel';
import FormsModel from './FormsModel';
import FormModelPublic from './public/FormModelPublic';

var FormModel = function FormModel(name, validator, onChange, parent) {
  var _this = this;

  _classCallCheck(this, FormModel);

  this.getPublic = function () {
    return _this['public'];
  };

  this.completed = function () {
    _this.initialized = true;
  };

  this.willDelete = function () {
    _this.initialized = false;
    _this.subscribers.unsubscribeAll();
    _this.onChange = function () {};
  };

  this.validate = function () {
    _this.fields.validate();
    _this.forms.validate();
    _this.refresh();
  };

  this.refresh = function () {
    var errors = _this.validator(_this.getPublic());
    _this.updateErrors(errors['']);
    _this.fields.addErrors(errors);
    _this.forms.addErrors(errors);
    _this.reload();
    _this.publish();
    if (_this.initialized) {
      _this.onChange(_this.getPublic());
    }
  };

  this.reload = function () {
    _this.fields.reload();
    _this.forms.reload();
  };

  this.reset = function () {
    _this.submitted = false;
    _this.fields.reset();
    _this.forms.reset();
    _this.validate();
  };

  this.init = function () {
    var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var makeDirty = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

    _this.submitted = false;
    _this.fields.reset();
    _this.forms.reset();
    _this.fields.init(data);
    _this.forms.init(data);
    if (makeDirty) {
      _this.fields.makeDirty();
      _this.forms.makeDirty();
    }
    _this.validate();
  };

  this.makeDirty = function () {
    _this.fields.makeDirty();
    _this.forms.makeDirty();
    _this.validate();
  };

  this.makeSubmitted = function () {
    _this.submitted = true;
    _this.fields.makeSubmitted();
    _this.forms.makeSubmitted();
    _this.reload();
    _this.publish();
  };

  this.isValid = function () {
    return !_this.errors.length && _this.fields.isValid() && _this.forms.isValid();
  };

  this.isTouched = function () {
    return _this.fields.isTouched() || _this.forms.isTouched();
  };

  this.isSubmitted = function () {
    return _this.submitted;
  };

  this.data = function () {
    return _extends({}, _this.fields.data(), _this.forms.data());
  };

  this.values = function () {
    return _extends({}, _this.fields.values(), _this.forms.values());
  };

  this.updateErrors = function (errors) {
    _this.errors = [];
    if (Array.isArray(errors)) {
      _this.errors = errors;
    } else if (errors) {
      _this.errors = [errors];
    }
  };

  this.getErrors = function () {
    return {
      fields: _this.fields.errors(),
      forms: _this.forms.errors(),
      form: _this.errors
    };
  };

  this.addSubscriber = function (fn) {
    var subscriberId = _this.subscribers.subscribe(fn);
    if (_this.initialized) {
      _this.publish();
    }
    return subscriberId;
  };

  this.removeSubscriber = function (id) {
    return _this.subscribers.unsubscribe(id);
  };

  this.publish = function () {
    _this.subscribers.publish();
  };

  this.id = uuid();
  this.parent = parent;
  this.name = name;
  this.validator = validator;
  this.initialized = false;
  this.submitted = false;
  this.subscribers = new PubSub();
  this.fields = new FieldsModel();
  this.forms = new FormsModel();
  this.errors = [];
  this.onChange = onChange || function () {};
  this['public'] = new FormModelPublic(this);
  this.refreshDebounce = debounce(this.refresh, 100);
};

export { FormModel };