import _extends from 'babel-runtime/helpers/extends';
import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import _possibleConstructorReturn from 'babel-runtime/helpers/possibleConstructorReturn';
import _inherits from 'babel-runtime/helpers/inherits';
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { isEqual } from 'lodash';
import { FormModel } from '../Form/FormModel';
import { FormItemModel } from '../Form/FormItemModel';
import { FieldErrors } from './FieldErrors';
import { getElementProps } from './getElementProps';

var FormField = function (_Component) {
  _inherits(FormField, _Component);

  function FormField(props, context) {
    _classCallCheck(this, FormField);

    var _this = _possibleConstructorReturn(this, _Component.call(this, props, context));

    _this.handleReload = function () {
      _this.forceUpdate();
      _this.props.onUpdate(_this.model.getPublic());
      _this.props.onFieldChange(_this.model.getPublic());
    };

    _this.onChange = function () {
      var value = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

      if (_this.model.submitted) {
        _this.model.changedAfterSubmit = true;
      }
      _this.model.setValue(value);
      _this.model.setDirty(true);
      _this.model.validate();
      _this.formModel.refresh();
      // this.formModel.updateFieldByName(this.model.name);
      _this.props.onChange(_this.model.getPublic());
      // this.props.onFieldChange(this.model, 'change');
    };

    _this.onFocus = function () {
      _this.model.setFocus(true);
      _this.model.validate();
      _this.formModel.refresh();
      //this.formModel.updateFieldByName(this.model.name);
      _this.props.onFocus(_this.model.getPublic());
      // this.props.onFieldChange(this.model, 'focus');
    };

    _this.onBlur = function () {
      _this.model.setFocus(false);
      _this.model.setTouched(true);
      _this.model.validate();
      _this.formModel.refresh();
      // this.formModel.updateFieldByName(this.model.name);
      _this.props.onBlur(_this.model.getPublic());
      // this.props.onFieldChange(this.model, 'blur');
    };

    var formModel = context.formModel,
        formItemModel = context.formItemModel;


    _this.fields = formModel.fields;
    if (formItemModel) {
      _this.fields = formItemModel.fields;
    }
    _this.model = _this.fields.add(_extends({ component: _this, reload: _this.handleReload }, props));
    _this.formModel = formModel;
    _this.formItemModel = formItemModel;
    return _this;
  }

  FormField.prototype.componentDidMount = function componentDidMount() {
    if (this.formModel.initialized) {
      this.formModel.validate();
    }
  };

  FormField.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
    // console.log(33333, nextProps);
    this.model.setSubmissionErrors(nextProps.submissionErrors);
    if (!isEqual(this.props.submissionErrors, nextProps.submissionErrors)) {
      if (this.formModel.initialized) {
        this.formModel.refreshDebounce();
      }
    }
  };

  FormField.prototype.shouldComponentUpdate = function shouldComponentUpdate(nextProps) {
    return !isEqual(getElementProps(nextProps), getElementProps(this.props));
  };

  FormField.prototype.componentWillUnmount = function componentWillUnmount() {
    this.fields.remove(this.model.name);
    if (this.formModel.initialized) {
      this.formModel.validate();
    }
  };

  FormField.prototype.render = function render() {
    var _props = this.props,
        component = _props.component,
        includeModel = _props.includeModel;


    var props = _extends({}, getElementProps(this.props), {
      value: this.model.value,
      onFocus: this.onFocus,
      onChange: this.onChange,
      onBlur: this.onBlur
    });
    if (includeModel) {
      props.model = this.model.getPublic();
    }
    return React.createElement(component, props);
  };

  return FormField;
}(Component);

FormField.propTypes = {
  onFocus: PropTypes.func,
  onChange: PropTypes.func,
  onBlur: PropTypes.func,
  onUpdate: PropTypes.func,
  onFieldChange: PropTypes.func
};
FormField.defaultProps = {
  onFocus: function onFocus() {},
  onChange: function onChange() {},
  onBlur: function onBlur() {},
  onUpdate: function onUpdate() {},
  onFieldChange: function onFieldChange() {}
};
FormField.contextTypes = {
  formModel: PropTypes.instanceOf(FormModel).isRequired, // TODO: need to investigate
  formItemModel: PropTypes.instanceOf(FormItemModel) // TODO: need to investigate
};


export { FormField, FieldErrors };